@@ -1,4 +1,4 @@
-use csml_engine::{fold_bot};
+use csml_engine::fold_bot;
 use csml_interpreter::{
     data::{csml_bot::CsmlBot, csml_flow::CsmlFlow},
     load_components,
@@ -19,7 +19,7 @@ fn init_bot() -> CsmlBot {
     CsmlBot {
         id: "botid".to_owned(),
         name: "plop".to_owned(),
-        fn_endpoint: Some("endpoint".to_owned()),
+        apps_endpoint: Some("endpoint".to_owned()),
         flows: vec![
             CsmlFlow {
                 id: "flowid".to_owned(),
@@ -38,7 +38,9 @@ fn init_bot() -> CsmlBot {
         custom_components: None,
         default_flow: "flow".to_owned(),
         bot_ast: None,
+        no_interruption_delay: None,
         env: None,
+        modules: None,
     }
 }
 
