@@ -4,6 +4,9 @@ use crate::db_connectors::{is_dynamodb, dynamodb_connector};
 use crate::db_connectors::{is_mongodb, mongodb_connector};
 #[cfg(feature = "postgresql")]
 use crate::db_connectors::{is_postgresql, postgresql_connector};
+#[cfg(feature = "sqlite")]
+use crate::db_connectors::{is_sqlite, sqlite_connector};
+
 
 use csml_interpreter::data::csml_logs::{LogLvl, CsmlLog, csml_logger};
 
@@ -65,6 +68,15 @@ pub fn create_conversation(
         );
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(db)?;
+        let expires_at = get_expires_at_for_sqlite(ttl);
+        return sqlite_connector::conversations::create_conversation(
+            flow_id, step_id, client, expires_at, db,
+        );
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
 
@@ -106,6 +118,12 @@ pub fn close_conversation(id: &str, client: &Client, db: &mut Database) -> Resul
         return postgresql_connector::conversations::close_conversation(id, client, "CLOSED", db);
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(db)?;
+        return sqlite_connector::conversations::close_conversation(id, client, "CLOSED", db);
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
 
@@ -147,6 +165,12 @@ pub fn close_all_conversations(client: &Client, db: &mut Database) -> Result<(),
         return postgresql_connector::conversations::close_all_conversations(client, db);
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(db)?;
+        return sqlite_connector::conversations::close_all_conversations(client, db);
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
 
@@ -191,6 +215,12 @@ pub fn get_latest_open(
         return postgresql_connector::conversations::get_latest_open(client, db);
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(db)?;
+        return sqlite_connector::conversations::get_latest_open(client, db);
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
 
@@ -253,6 +283,17 @@ pub fn update_conversation(
         );
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(&mut data.db)?;
+        return sqlite_connector::conversations::update_conversation(
+            &data.conversation_id,
+            flow_id,
+            step_id,
+            db,
+        );
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
 
@@ -318,5 +359,16 @@ pub fn get_client_conversations(
         );
     }
 
+    #[cfg(feature = "sqlite")]
+    if is_sqlite() {
+        let db = sqlite_connector::get_db(db)?;
+        return sqlite_connector::conversations::get_client_conversations(
+            client,
+            db,
+            limit,
+            pagination_key
+        );
+    }
+
     Err(EngineError::Manager(ERROR_DB_SETUP.to_owned()))
 }
