@@ -1,22 +1,18 @@
 use crate::data::DynamoDbClient;
 use crate::{Client, Database, EngineError};
-use serde::{Deserialize, Serialize};
-use uuid::Uuid;
-use std::collections::HashMap;
 use rusoto_dynamodb::AttributeValue;
+use serde::{Deserialize, Serialize};
+use std::collections::HashMap;
+use uuid::Uuid;
 
 pub mod aws_s3;
 pub mod bot;
 pub mod conversations;
-pub mod interactions;
 pub mod memories;
 pub mod messages;
-pub mod nodes;
 pub mod state;
 pub mod utils;
 
-mod dynamodb_tests;
-
 use crate::db_connectors::dynamodb::utils::*;
 
 use rusoto_core::Region;
@@ -59,7 +55,9 @@ pub fn get_db<'a>(db: &'a mut Database) -> Result<&'a mut DynamoDbClient, Engine
     }
 }
 
-pub fn get_pagination_key(pagination_key: Option<String>) ->  Result<Option<HashMap<String, AttributeValue>>, EngineError> {
+pub fn get_pagination_key(
+    pagination_key: Option<String>,
+) -> Result<Option<HashMap<String, AttributeValue>>, EngineError> {
     match pagination_key {
         Some(key) => {
             let base64decoded = match base64::decode(&key) {
@@ -133,7 +131,6 @@ impl Bot {
     }
 }
 
-
 #[derive(Serialize, Deserialize, Debug)]
 pub struct ConversationDeleteInfo {
     pub status: String,
@@ -155,6 +152,7 @@ pub struct Conversation {
     pub step_id: String,
     pub status: String,
     pub last_interaction_at: String,
+    pub expires_at: Option<i64>,
     pub updated_at: String,
     pub created_at: String,
 }
@@ -168,12 +166,6 @@ impl Conversation {
         make_range(&["conversation", status, id])
     }
 
-    pub fn get_conversation_id_from_range(range: &str) -> String {
-        let vec: Vec<&str> = range.split("#").collect();
-
-        vec[2].to_owned()
-    }
-
     pub fn get_key(client: &Client, status: &str, id: &str) -> DynamoDbKey {
         let hash = Self::get_hash(client);
         let range = Self::get_range(status, id);
@@ -185,7 +177,7 @@ impl Conversation {
      * range = conversation#OPEN|CLOSED#id
      * range_time = conversation#OPEN|CLOSED#timestamp#id
      */
-    pub fn new(client: &Client, flow_id: &str, step_id: &str) -> Self {
+    pub fn new(client: &Client, flow_id: &str, step_id: &str, expires_at: Option<i64>) -> Self {
         let id = Uuid::new_v4().to_string();
         let now = get_date_time();
         let status = "OPEN";
@@ -204,70 +196,7 @@ impl Conversation {
             step_id: step_id.to_owned(),
             status: status.to_owned(),
             last_interaction_at: now.to_owned(),
-            updated_at: now.to_owned(),
-            created_at: now.to_owned(),
-        }
-    }
-}
-
-#[derive(Serialize, Deserialize, Debug)]
-pub struct InteractionDeleteInfo {
-    pub id: String,
-}
-
-#[derive(Serialize, Deserialize, Debug)]
-pub struct Interaction {
-    pub hash: String,
-    pub range: String,
-    pub range_time: String,
-    pub class: String,
-    pub id: String,
-    pub client: Option<Client>,
-    pub bot_id: Option<String>,
-    pub channel_id: Option<String>,
-    pub user_id: Option<String>,
-    pub success: bool,
-    pub event: String,
-    pub updated_at: String,
-    pub created_at: String,
-}
-
-impl Interaction {
-    pub fn get_hash(client: &Client) -> String {
-        make_hash(client)
-    }
-
-    pub fn get_range(id: &str) -> String {
-        make_range(&["interaction", id])
-    }
-
-    pub fn get_key(client: &Client, id: &str) -> DynamoDbKey {
-        let hash = Self::get_hash(client);
-        let range = Self::get_range(id);
-        DynamoDbKey::new(&hash, &range)
-    }
-
-    /**
-     * hash = bot_id:xxxx#channel_id:xxxx#user_id:xxxx
-     * range = interaction#id
-     * range_time = interaction#timestamp#id
-     */
-    pub fn new(id: &Uuid, client: &Client, encrypted_event: &str) -> Self {
-        let class_name = "interaction";
-        let now = get_date_time();
-        let id = id.to_string();
-        Self {
-            hash: Self::get_hash(client),
-            range: Self::get_range(&id),
-            range_time: make_range(&[class_name, &now, &id]),
-            class: class_name.to_string(),
-            id: id.to_owned(),
-            client: Some(client.clone()),
-            bot_id: Some(client.bot_id.to_owned()),
-            channel_id: Some(client.channel_id.to_owned()),
-            user_id: Some(client.user_id.to_owned()),
-            success: false,
-            event: encrypted_event.to_owned(),
+            expires_at,
             updated_at: now.to_owned(),
             created_at: now.to_owned(),
         }
@@ -297,7 +226,7 @@ pub struct Memory {
     pub user_id: Option<String>,
     pub key: String,
     pub value: Option<String>,
-    pub expires_at: Option<String>,
+    pub expires_at: Option<i64>,
     pub created_at: String,
 }
 
@@ -319,6 +248,7 @@ impl Memory {
         client: &Client,
         key: &str,
         encrypted_value: Option<String>,
+        expires_at: Option<i64>,
     ) -> Self {
         let hash = Self::get_hash(client);
         let range = Self::get_range(key);
@@ -328,11 +258,7 @@ impl Memory {
         Self {
             hash: hash.to_owned(),
             range: range.to_owned(),
-            range_time: make_range(&[
-                class_name,
-                &now,
-                &range
-            ]),
+            range_time: make_range(&[class_name, &now, &range]),
             class: class_name.to_owned(),
             client: Some(client.to_owned()),
             bot_id: Some(client.bot_id.to_owned()),
@@ -340,19 +266,26 @@ impl Memory {
             user_id: Some(client.user_id.to_owned()),
             key: key.to_owned(),
             value: encrypted_value.clone(),
-            expires_at: None,
+            expires_at,
             created_at: now.to_owned(),
         }
     }
 }
 
-
 #[derive(Serialize, Deserialize, Debug)]
 struct MessageDeleteInfo {
     conversation_id: String,
     id: String,
 }
 
+#[derive(Serialize, Deserialize, Debug)]
+struct MessageFromDateInfo {
+    class: String,
+    range: String,
+    created_at: String,
+    hash: String,
+}
+
 #[derive(Serialize, Deserialize, Debug)]
 pub struct Message {
     pub hash: String,
@@ -364,7 +297,6 @@ pub struct Message {
     pub bot_id: Option<String>,
     pub channel_id: Option<String>,
     pub user_id: Option<String>,
-    pub interaction_id: String,
     pub conversation_id: String,
     pub flow_id: String,
     pub step_id: String,
@@ -373,9 +305,57 @@ pub struct Message {
     pub direction: String,
     pub payload: String,
     pub content_type: String,
+    pub expires_at: Option<i64>,
     pub created_at: String,
 }
 
+// {
+//     "class": AttributeValue {
+//         b: None,
+//         bool: None,
+//         bs: None,
+//         l: None,
+//         m: None,
+//         n: None,
+//         ns: None,
+//         null: None,
+//         s: Some("message"), ss: None
+//     },
+//     "range": AttributeValue {
+//             b: None,
+//             bool: None,
+//             bs: None,
+//             l: None,
+//             m: None,
+//             n: None,
+//             ns: None,
+//             null: None,
+//             s: Some("message#8ced58a8-ae25-47bd-a949-4d702f7650b9#a649e6d1.-c4fa-44db-950e-3dbfafb8ce7b"), ss: None
+//     },
+//     "created_at": AttributeValue {
+//         b: None,
+//         bool: None,
+//         bs: None,
+//         l: None,
+//         m: None,
+//         n: None,
+//         ns: None,
+//         null: None,
+//         s: Some("2022-04-08T13:52:29.841Z"), ss: None
+//     },
+//     "hash": AttributeValue {
+//         b: None,
+//         bool: None,
+//         bs: None,
+//         l: None,
+//         m: None,
+//         n: None,
+//         ns: None,
+//         null: None,
+//         s: Some("bot_id:botid#channel_id:some-channel-id#user_id:alexis"), ss: None
+//     }
+// }
+
 impl Message {
     pub fn get_hash(client: &Client) -> String {
         make_hash(client)
@@ -393,7 +373,6 @@ impl Message {
     pub fn new(
         client: &Client,
         conversation_id: &str,
-        interaction_id: &str,
         flow_id: &str,
         step_id: &str,
         direction: &str,
@@ -401,6 +380,7 @@ impl Message {
         message_order: i32,
         payload: &str,
         content_type: &str,
+        expires_at: Option<i64>,
     ) -> Self {
         let id = uuid::Uuid::new_v4().to_string();
         let class_name = "message";
@@ -421,7 +401,6 @@ impl Message {
             bot_id: Some(client.bot_id.to_owned()),
             channel_id: Some(client.channel_id.to_owned()),
             user_id: Some(client.user_id.to_owned()),
-            interaction_id: interaction_id.to_owned(),
             conversation_id: conversation_id.to_owned(),
             flow_id: flow_id.to_owned(),
             step_id: step_id.to_owned(),
@@ -430,71 +409,7 @@ impl Message {
             direction: direction.to_owned(),
             payload: payload.to_owned(),
             content_type: content_type.to_owned(),
-            created_at: now.to_owned(),
-        }
-    }
-}
-
-#[derive(Serialize, Deserialize, Debug)]
-pub struct NodeDeleteInfo {
-    pub id: String,
-    pub conversation_id: String,
-}
-
-#[derive(Serialize, Deserialize, Debug)]
-pub struct Node {
-    pub hash: String,
-    pub range: String,
-    pub range_time: String,
-    pub class: String,
-    pub id: String,
-    pub client: Option<Client>,
-    pub bot_id: Option<String>,
-    pub channel_id: Option<String>,
-    pub user_id: Option<String>,
-    pub conversation_id: String,
-    pub interaction_id: String,
-    pub flow_id: String,
-    pub step_id: String,
-    pub next_flow: Option<String>,
-    pub next_step: Option<String>,
-    pub created_at: String,
-}
-
-impl Node {
-    /**
-     * hash = conversation:xxxx
-     * range = path#id
-     * range_time = path#timestamp#id
-     */
-    pub fn new(
-        client: &Client,
-        conversation_id: &str,
-        interaction_id: &str,
-        flow_id: &str,
-        step_id: &str,
-        next_flow: Option<String>,
-        next_step: Option<String>,
-    ) -> Self {
-        let id = uuid::Uuid::new_v4().to_string();
-        let class_name = "path";
-        let now = get_date_time();
-        Self {
-            hash: make_range(&["conversation", conversation_id]),
-            range: make_range(&[class_name, &id]),
-            range_time: make_range(&[class_name, &now, &id]),
-            class: class_name.to_owned(),
-            id: id.to_owned(),
-            client: Some(client.to_owned()),
-            bot_id: Some(client.bot_id.to_owned()),
-            channel_id: Some(client.channel_id.to_owned()),
-            user_id: Some(client.user_id.to_owned()),
-            conversation_id: conversation_id.to_owned(),
-            interaction_id: interaction_id.to_owned(),
-            flow_id: flow_id.to_owned(),
-            step_id: step_id.to_owned(),
-            next_flow: next_flow.clone(),
-            next_step: next_step.clone(),
+            expires_at,
             created_at: now.to_owned(),
         }
     }
@@ -521,7 +436,7 @@ pub struct State {
     pub _type: String,
     pub key: String,
     pub value: String,
-    pub expires_at: Option<String>,
+    pub expires_at: Option<i64>,
     pub created_at: String,
 }
 
@@ -538,7 +453,13 @@ impl State {
      * hash = bot_id:xxxx#channel_id:xxxx#user_id:xxxx
      * range = mem_state#id
      */
-    pub fn new(client: &Client, _type: &str, key: &str, encrypted_value: &str) -> Self {
+    pub fn new(
+        client: &Client,
+        _type: &str,
+        key: &str,
+        encrypted_value: &str,
+        expires_at: Option<i64>,
+    ) -> Self {
         let class_name = "state";
         let id = uuid::Uuid::new_v4().to_string();
         let now = get_date_time();
@@ -554,7 +475,7 @@ impl State {
             _type: _type.to_string(),
             key: key.to_owned(),
             value: encrypted_value.to_owned(),
-            expires_at: None,
+            expires_at,
             created_at: now.to_string(),
         }
     }
@@ -565,4 +486,4 @@ pub struct Class {
     pub class: String,
     pub hash: String,
     pub range: String,
-}
\ No newline at end of file
+}
