@@ -1,9 +1,9 @@
 use crate::{
+    data::{CsmlBotBincode, MongoDbClient, SerializeCsmlBot},
     db_connectors::{BotVersion, DbBot},
-    data::{SerializeCsmlBot, CsmlBotBincode, MongoDbClient},
     EngineError,
 };
-use bson::{doc, Bson};
+use bson::{doc, Document};
 use chrono::SecondsFormat;
 
 fn format_bot_struct(bot: bson::document::Document) -> Result<DbBot, EngineError> {
@@ -15,6 +15,7 @@ fn format_bot_struct(bot: bson::document::Document) -> Result<DbBot, EngineError
         created_at: bot
             .get_datetime("created_at")
             .unwrap()
+            .to_chrono()
             .to_rfc3339_opts(SecondsFormat::Millis, true),
     })
 }
@@ -24,8 +25,8 @@ pub fn create_bot_version(
     bot: String,
     db: &MongoDbClient,
 ) -> Result<String, EngineError> {
-    let collection = db.client.collection("bot");
-    let time = Bson::DateTime(chrono::Utc::now());
+    let collection = db.client.collection::<Document>("bot");
+    let time = bson::DateTime::from_chrono(chrono::Utc::now());
 
     let bot = doc! {
         "bot_id": bot_id,
@@ -47,19 +48,18 @@ pub fn get_bot_versions(
     pagination_key: Option<String>,
     db: &MongoDbClient,
 ) -> Result<serde_json::Value, EngineError> {
-    let collection = db.client.collection("bot");
+    let collection = db.client.collection::<Document>("bot");
 
     let limit = match limit {
-        Some(limit) if limit >= 1 => limit + 1,
-        Some(_limit) => 21,
-        None => 21,
+        Some(limit) => std::cmp::min(limit + 1 , 26),
+        None => 26,
     };
 
     let filter = match pagination_key {
         Some(key) => {
             doc! {
                 "bot_id": bot_id,
-                "_id": {"$gt": bson::oid::ObjectId::with_string(&key).unwrap() }
+                "_id": {"$gt": bson::oid::ObjectId::parse_str(&key).unwrap() }
             }
         }
         None => doc! {"bot_id": bot_id },
@@ -80,13 +80,13 @@ pub fn get_bot_versions(
                 let bot_version = format_bot_struct(bot_doc)?;
 
                 let csml_bot: SerializeCsmlBot = match base64::decode(&bot_version.bot) {
-                    Ok(base64decoded) =>  {
+                    Ok(base64decoded) => {
                         match bincode::deserialize::<CsmlBotBincode>(&base64decoded[..]) {
                             Ok(bot) => bot.to_bot(),
-                            Err(_) => serde_json::from_str(&bot_version.bot).unwrap()
+                            Err(_) => serde_json::from_str(&bot_version.bot).unwrap(),
                         }
-                    },
-                    Err(_) => serde_json::from_str(&bot_version.bot).unwrap()
+                    }
+                    Err(_) => serde_json::from_str(&bot_version.bot).unwrap(),
                 };
 
                 let mut json = serde_json::json!({
@@ -128,10 +128,10 @@ pub fn get_bot_by_version_id(
     id: &str,
     db: &MongoDbClient,
 ) -> Result<Option<BotVersion>, EngineError> {
-    let collection = db.client.collection("bot");
+    let collection = db.client.collection::<Document>("bot");
 
     let filter = doc! {
-        "_id": bson::oid::ObjectId::with_string(id).unwrap()
+        "_id": bson::oid::ObjectId::parse_str(id).unwrap()
     };
 
     let find_options = mongodb::options::FindOneOptions::builder()
@@ -145,13 +145,13 @@ pub fn get_bot_by_version_id(
             let bot = format_bot_struct(bot)?;
 
             let csml_bot: SerializeCsmlBot = match base64::decode(&bot.bot) {
-                Ok(base64decoded) =>  {
+                Ok(base64decoded) => {
                     match bincode::deserialize::<CsmlBotBincode>(&base64decoded[..]) {
                         Ok(bot) => bot.to_bot(),
-                        Err(_) => serde_json::from_str(&bot.bot).unwrap()
+                        Err(_) => serde_json::from_str(&bot.bot).unwrap(),
                     }
-                },
-                Err(_) => serde_json::from_str(&bot.bot).unwrap()
+                }
+                Err(_) => serde_json::from_str(&bot.bot).unwrap(),
             };
 
             Ok(Some(BotVersion {
@@ -168,7 +168,7 @@ pub fn get_last_bot_version(
     bot_id: &str,
     db: &MongoDbClient,
 ) -> Result<Option<BotVersion>, EngineError> {
-    let collection = db.client.collection("bot");
+    let collection = db.client.collection::<Document>("bot");
 
     let filter = doc! {
         "bot_id": bot_id,
@@ -188,10 +188,10 @@ pub fn get_last_bot_version(
                 Ok(base64decoded) => {
                     match bincode::deserialize::<CsmlBotBincode>(&base64decoded[..]) {
                         Ok(bot) => bot.to_bot(),
-                        Err(_) => serde_json::from_str(&bot.bot).unwrap()
+                        Err(_) => serde_json::from_str(&bot.bot).unwrap(),
                     }
-                },
-                Err(_) => serde_json::from_str(&bot.bot).unwrap()
+                }
+                Err(_) => serde_json::from_str(&bot.bot).unwrap(),
             };
 
             Ok(Some(BotVersion {
@@ -205,10 +205,10 @@ pub fn get_last_bot_version(
 }
 
 pub fn delete_bot_version(version_id: &str, db: &MongoDbClient) -> Result<(), EngineError> {
-    let collection = db.client.collection("bot");
+    let collection = db.client.collection::<Document>("bot");
 
     let filter = doc! {
-        "_id": bson::oid::ObjectId::with_string(version_id).unwrap()
+        "_id": bson::oid::ObjectId::parse_str(version_id).unwrap()
     };
 
     collection.delete_one(filter, None)?;
@@ -217,7 +217,7 @@ pub fn delete_bot_version(version_id: &str, db: &MongoDbClient) -> Result<(), En
 }
 
 pub fn delete_bot_versions(bot_id: &str, db: &MongoDbClient) -> Result<(), EngineError> {
-    let collection = db.client.collection("bot");
+    let collection = db.client.collection::<Document>("bot");
 
     let filter = doc! {
         "bot_id": bot_id,
@@ -228,9 +228,12 @@ pub fn delete_bot_versions(bot_id: &str, db: &MongoDbClient) -> Result<(), Engin
     Ok(())
 }
 
-pub fn delete_all_bot_data(bot_id: &str, class: &str, db: &MongoDbClient) -> Result<(), EngineError> {
-
-    let collection = db.client.collection(class);
+pub fn delete_all_bot_data(
+    bot_id: &str,
+    class: &str,
+    db: &MongoDbClient,
+) -> Result<(), EngineError> {
+    let collection = db.client.collection::<Document>(class);
 
     let filter = doc! {
         "client.bot_id": bot_id,
@@ -239,4 +242,4 @@ pub fn delete_all_bot_data(bot_id: &str, class: &str, db: &MongoDbClient) -> Res
     collection.delete_many(filter, None)?;
 
     Ok(())
-}
\ No newline at end of file
+}
