@@ -1,21 +1,22 @@
 use crate::{
-    db_connectors::{mongodb::{get_db}, DbMessage},
-    encrypt::{encrypt_data, decrypt_data},
-    ConversationInfo, EngineError, Client,
-    MongoDbClient
+    db_connectors::{mongodb::get_db, DbMessage},
+    encrypt::{decrypt_data, encrypt_data},
+    Client, ConversationInfo, EngineError, MongoDbClient,
 };
-use bson::{doc, Bson, Document};
+use bson::{doc, Document};
+use chrono::SecondsFormat;
 
 fn format_messages(
     data: &ConversationInfo,
     messages: &[serde_json::Value],
     interaction_order: i32,
     direction: &str,
+    expires_at: Option<bson::DateTime>,
 ) -> Result<Vec<Document>, EngineError> {
     messages
         .iter()
         .enumerate()
-        .map(|(i, var)| format_message(data, var.clone(), i as i32, interaction_order, direction))
+        .map(|(i, var)| format_message(data, var.clone(), i as i32, interaction_order, direction, expires_at))
         .collect::<Result<Vec<Document>, EngineError>>()
 }
 
@@ -25,11 +26,11 @@ fn format_message(
     msg_order: i32,
     interaction_order: i32,
     direction: &str,
+    expires_at: Option<bson::DateTime>,
 ) -> Result<Document, EngineError> {
-    let time = Bson::DateTime(chrono::Utc::now());
+    let time = bson::DateTime::from_chrono(chrono::Utc::now());
     let doc = doc! {
         "client": bson::to_bson(&data.client)?,
-        "interaction_id": &data.interaction_id,
         "conversation_id": &data.conversation_id,
         "flow_id": &data.context.flow,
         "step_id": &data.context.step,
@@ -37,23 +38,20 @@ fn format_message(
         "interaction_order": interaction_order,
         "direction": direction,
         "payload": encrypt_data(&message)?, // encrypted
-        "content_type": &message["content_type"].as_str().unwrap_or("text"),
+        "expires_at": expires_at,
         "created_at": time
     };
 
     Ok(doc)
 }
 
-fn format_message_struct(
-    message: bson::document::Document,
-) -> Result<DbMessage, EngineError> {
+fn format_message_struct(message: bson::document::Document) -> Result<DbMessage, EngineError> {
     let encrypted_payload: String = message.get_str("payload").unwrap().to_owned();
     let payload = decrypt_data(encrypted_payload)?;
 
     Ok(DbMessage {
         id: message.get_object_id("_id").unwrap().to_hex(), // to_hex bson::oid::ObjectId
         client: bson::from_bson(message.get("client").unwrap().to_owned())?,
-        interaction_id: message.get_str("interaction_id").unwrap().to_owned(), 
         conversation_id: message.get_str("conversation_id").unwrap().to_owned(),
         flow_id: message.get_str("flow_id").unwrap().to_owned(),
         step_id: message.get_str("step_id").unwrap().to_owned(),
@@ -61,8 +59,7 @@ fn format_message_struct(
         interaction_order: message.get_i32("interaction_order").unwrap(),
         direction: message.get_str("direction").unwrap().to_owned(),
         payload,
-        content_type: message.get_str("content_type").unwrap().to_owned(),
-        created_at: message.get_str("created_at").unwrap().to_owned(),
+        created_at: message.get_datetime("created_at").unwrap().to_chrono().to_rfc3339_opts(SecondsFormat::Millis, true),
     })
 }
 
@@ -71,14 +68,15 @@ pub fn add_messages_bulk(
     msgs: &[serde_json::Value],
     interaction_order: i32,
     direction: &str,
+    expires_at: Option<bson::DateTime>,
 ) -> Result<(), EngineError> {
     if msgs.len() == 0 {
         return Ok(());
     }
-    let docs = format_messages(data, msgs, interaction_order, direction)?;
+    let docs = format_messages(data, msgs, interaction_order, direction, expires_at)?;
     let db = get_db(&data.db)?;
 
-    let message = db.client.collection("message");
+    let message = db.client.collection::<Document>("message");
 
     message.insert_many(docs, None)?;
 
@@ -86,7 +84,7 @@ pub fn add_messages_bulk(
 }
 
 pub fn delete_user_messages(client: &Client, db: &MongoDbClient) -> Result<(), EngineError> {
-    let collection = db.client.collection("message");
+    let collection = db.client.collection::<Document>("message");
 
     let filter = doc! {
         "client": bson::to_bson(&client)?,
@@ -102,23 +100,49 @@ pub fn get_client_messages(
     db: &MongoDbClient,
     limit: Option<i64>,
     pagination_key: Option<String>,
+    from_date: Option<i64>,
+    to_date: Option<i64>,
 ) -> Result<serde_json::Value, EngineError> {
-    let collection = db.client.collection("conversation");
+    let collection = db.client.collection::<Document>("message");
 
     let limit = match limit {
-        Some(limit) if limit >= 1 => limit + 1,
-        Some(_limit) => 21,
-        None => 21,
+        Some(limit) => std::cmp::min(limit + 1 , 26),
+        None => 26,
     };
 
-    let filter = match pagination_key {
-        Some(key) => {
+    let filter = match (pagination_key, from_date) {
+        (Some(key), Some(from_date)) => {
+            let from_date = bson::DateTime::from_millis(from_date * 1000);
+            let to_date = match to_date {
+                Some(to_date) => bson::DateTime::from_millis(to_date * 1000),
+                None => bson::DateTime::from_chrono(chrono::Utc::now())
+            };
+
             doc! {
                 "client": bson::to_bson(&client)?,
-                "_id": {"$gt": bson::oid::ObjectId::with_string(&key).unwrap() }
+                "_id": {"$gt": bson::oid::ObjectId::parse_str(&key).unwrap() },
+                "created_at": {"$gte": from_date, "$lt": to_date}
             }
         }
-        None => doc! {
+        (None, Some(from_date)) => {
+            let from_date = bson::DateTime::from_millis(from_date * 1000);
+            let to_date = match to_date {
+                Some(to_date) => bson::DateTime::from_millis(to_date * 1000),
+                None => bson::DateTime::from_chrono(chrono::Utc::now())
+            };
+
+            doc! {
+                "client": bson::to_bson(&client)?,
+                "created_at": {"$gte": from_date, "$lt": to_date}
+            }
+        }
+        (Some(key), None) => {
+            doc! {
+                "client": bson::to_bson(&client)?,
+                "_id": {"$gt": bson::oid::ObjectId::parse_str(&key).unwrap() },
+            }
+        }
+        (None, None) => doc! {
             "client": bson::to_bson(&client)?,
         },
     };
@@ -128,6 +152,7 @@ pub fn get_client_messages(
         .batch_size(30)
         .limit(limit)
         .build();
+
     let cursor = collection.find(filter, find_options)?;
 
     let mut messages = vec![];
@@ -138,13 +163,11 @@ pub fn get_client_messages(
 
                 let json = serde_json::json!({
                     "client": message.client,
-                    "interaction_id": message.interaction_id,
                     "conversation_id": message.conversation_id,
                     "flow_id": message.flow_id,
                     "step_id": message.step_id,
                     "direction": message.direction,
                     "payload": message.payload,
-                    "content_type": message.content_type,
                     "created_at": message.created_at,
                 });
 
@@ -168,4 +191,4 @@ pub fn get_client_messages(
         }
         false => Ok(serde_json::json!({ "messages": messages })),
     }
-}
\ No newline at end of file
+}
