@@ -1,20 +1,20 @@
-use crate::db_connectors::{conversations::*, interactions::*, memories::*};
+use crate::db_connectors::{conversations::*, memories::*};
 use crate::{
     data::{ConversationInfo, CsmlRequest, Database, EngineError},
-    utils::{get_default_flow, get_flow_by_id, search_flow},
+    utils::{
+        get_default_flow, get_flow_by_id, get_low_data_mode_value, get_ttl_duration_value,
+        search_flow,
+    },
     Context, CsmlBot, CsmlFlow, CsmlResult,
 };
 
 use csml_interpreter::{
     data::{
+        ast::Flow,
         context::{get_hashmap_from_json, get_hashmap_from_mem},
         ApiInfo, Client, Event,
     },
-    load_components, validate_bot,
-};
-use curl::{
-    easy::{Easy, List},
-    Error as CurlError,
+    load_components, search_for_modules, validate_bot,
 };
 use std::collections::HashMap;
 
@@ -41,25 +41,10 @@ pub fn init_conversation_info<'a>(
 ) -> Result<ConversationInfo, EngineError> {
     // Create a new interaction. An interaction is basically each request,
     // initiated from the bot or the user.
-    let interaction_id = init_interaction(request.payload.clone(), &request.client, &mut db)?;
-    let mut context = init_context(default_flow, request.client.clone(), &bot.fn_endpoint);
 
-    // Create and cache a curl agent to call the callback_url for every new message.
-    // If no callback_url is set, no message will be sent as they are processed and
-    // they will only be returned at the end of the fully-processed and successful request.
-    let curl = match request.callback_url {
-        Some(ref url) => {
-            if let Ok(curl) = init_curl(url) {
-                Some(curl)
-            } else {
-                return Err(EngineError::Manager(format!(
-                    "not valid callback_url {}",
-                    url
-                )));
-            }
-        }
-        None => None,
-    };
+    let mut context = init_context(default_flow, request.client.clone(), &bot.apps_endpoint);
+    let ttl = get_ttl_duration_value(Some(event));
+    let low_data = get_low_data_mode_value(event);
 
     // Do we have a flow matching the request? If the user is requesting a flow in one way
     // or another, this takes precedence over any previously open conversation
@@ -70,21 +55,26 @@ pub fn init_conversation_info<'a>(
         &bot,
         flow_found,
         &request.client,
+        ttl,
         &mut db,
     )?;
 
     context.metadata = get_hashmap_from_json(&request.metadata, &context.flow);
-    context.current = get_hashmap_from_mem(&internal_use_get_memories(&request.client, &mut db)?, &context.flow);
+    context.current = get_hashmap_from_mem(
+        &internal_use_get_memories(&request.client, &mut db)?,
+        &context.flow,
+    );
 
     let mut data = ConversationInfo {
         conversation_id,
-        interaction_id,
         context,
         metadata: request.metadata.clone(), // ??
         request_id: request.request_id.clone(),
-        curl,
+        callback_url: request.callback_url.clone(),
         client: request.client.clone(),
         messages: vec![],
+        ttl,
+        low_data,
         db,
     };
 
@@ -108,13 +98,32 @@ pub fn init_bot(bot: &mut CsmlBot) -> Result<(), EngineError> {
         Err(err) => return Err(EngineError::Interpreter(err.format_error())),
     };
 
+    if let Err(err) = search_for_modules(bot) {
+        return Err(EngineError::Interpreter(format!("{:?}", err)));
+    }
+
     match validate_bot(&bot) {
         CsmlResult {
             flows: Some(flows),
+            extern_flows: Some(extern_flows),
             errors: None,
             ..
         } => {
-            bot.bot_ast = Some(base64::encode(bincode::serialize(&flows).unwrap()));
+            bot.bot_ast = Some(base64::encode(
+                bincode::serialize(&(&flows, &extern_flows)).unwrap(),
+            ));
+        }
+        CsmlResult {
+            flows: Some(flows),
+            extern_flows: None,
+            errors: None,
+            ..
+        } => {
+            let extern_flows: HashMap<String, Flow> = HashMap::new();
+
+            bot.bot_ast = Some(base64::encode(
+                bincode::serialize(&(&flows, &extern_flows)).unwrap(),
+            ));
         }
         CsmlResult {
             errors: Some(errors),
@@ -134,11 +143,11 @@ pub fn init_bot(bot: &mut CsmlBot) -> Result<(), EngineError> {
 /**
  * Initialize the context object for incoming requests
  */
-pub fn init_context(flow: String, client: Client, fn_endpoint: &Option<String>) -> Context {
-    let api_info = match fn_endpoint {
+pub fn init_context(flow: String, client: Client, apps_endpoint: &Option<String>) -> Context {
+    let api_info = match apps_endpoint {
         Some(value) => Some(ApiInfo {
             client,
-            fn_endpoint: value.to_owned(),
+            apps_endpoint: value.to_owned(),
         }),
         None => None,
     };
@@ -153,37 +162,22 @@ pub fn init_context(flow: String, client: Client, fn_endpoint: &Option<String>)
     }
 }
 
-/**
- * Initialize a curl agent for standardized post requests to a given url.
- * It should be cached whenever possible to reuse existing connections.
- */
-pub fn init_curl(url: &str) -> Result<Easy, CurlError> {
-    let mut easy = Easy::new();
-    let mut list = List::new();
-    easy.url(url)?;
-    easy.post(true)?;
-
-    list.append("Accept: application/json")?;
-    list.append("Content-Type: application/json")?;
-    easy.http_headers(list)?;
-    Ok(easy)
-}
-
 /**
  * Retrieve the current conversation, or create one if none exists.
  */
 fn get_or_create_conversation<'a>(
     context: &mut Context,
     bot: &'a CsmlBot,
-    flow_found: Option<&'a CsmlFlow>,
+    flow_found: Option<(&'a CsmlFlow, String)>,
     client: &Client,
+    ttl: Option<chrono::Duration>,
     db: &mut Database,
 ) -> Result<String, EngineError> {
     match get_latest_open(client, db)? {
         Some(conversation) => {
             match flow_found {
-                Some(flow) => {
-                    context.step = "start".to_owned();
+                Some((flow, step)) => {
+                    context.step = step;
                     context.flow = flow.name.to_owned();
                 }
                 None => {
@@ -194,7 +188,7 @@ fn get_or_create_conversation<'a>(
                             close_conversation(&conversation.id, &client, db)?;
                             // start new conversation at default flow
                             return create_new_conversation(
-                                context, bot, flow_found, client, db,
+                                context, bot, flow_found, client, ttl, db,
                             );
                         }
                     };
@@ -206,7 +200,7 @@ fn get_or_create_conversation<'a>(
 
             Ok(conversation.id)
         }
-        None => create_new_conversation(context, bot, flow_found, client, db),
+        None => create_new_conversation(context, bot, flow_found, client, ttl, db),
     }
 }
 
@@ -216,19 +210,19 @@ fn get_or_create_conversation<'a>(
 fn create_new_conversation<'a>(
     context: &mut Context,
     bot: &'a CsmlBot,
-    flow_found: Option<&'a CsmlFlow>,
+    flow_found: Option<(&'a CsmlFlow, String)>,
     client: &Client,
+    ttl: Option<chrono::Duration>,
     db: &mut Database,
 ) -> Result<String, EngineError> {
-    let flow = match flow_found {
-        Some(flow) => flow,
-        None => get_default_flow(bot)?,
+    let (flow, step) = match flow_found {
+        Some((flow, step)) => (flow, step),
+        None => (get_default_flow(bot)?, "start".to_owned()),
     };
-    context.step = "start".to_owned();
+    context.step = step;
     context.flow = flow.name.to_owned();
 
-    let conversation_id =
-        create_conversation(&flow.id, &context.step, client, db)?;
+    let conversation_id = create_conversation(&flow.id, &context.step, client, ttl, db)?;
 
     Ok(conversation_id)
 }
