@@ -2,37 +2,39 @@ pub mod data;
 pub use csml_interpreter::{
     data::{
         ast::{Expr, Flow, InstructionScope},
-        error_info::ErrorInfo,
-        warnings::Warnings,
-        Client, CsmlResult,
         csml_logs::*,
+        csml_logs::*,
+        error_info::ErrorInfo,
+        position::Position,
+        warnings::Warnings,
+        Client, CsmlResult, Event,
     },
-    load_components,
+    load_components, search_for_modules,
 };
 
 use serde_json::json;
 
-mod error_messages;
 mod db_connectors;
 mod encrypt;
+mod error_messages;
 mod init;
 mod interpreter_actions;
 mod send;
 mod utils;
 
-#[cfg(feature = "postgresql")]
+#[cfg(any(feature = "postgresql", feature = "sqlite"))]
 #[macro_use]
 extern crate diesel;
 
-#[cfg(feature = "postgresql")]
+#[cfg(any(feature = "postgresql", feature = "sqlite"))]
 #[macro_use]
 extern crate diesel_migrations;
 
 use data::*;
 use db_connectors::{
-    bot, conversations, init_db, memories, messages, state, user,
-    state::{set_state_items, delete_state_key}, clean_db, BotVersion, BotVersionCreated,
-    DbConversation,
+    bot, clean_db, conversations, init_db, memories, messages, state,
+    state::{delete_state_key, set_state_items},
+    user, BotVersion, BotVersionCreated, DbConversation,
 };
 use init::*;
 use interpreter_actions::interpret_step;
@@ -63,7 +65,7 @@ pub fn start_conversation(
 ) -> Result<serde_json::Map<String, serde_json::Value>, EngineError> {
     init_logger();
 
-    let formatted_event = format_event(json!(request))?;
+    let mut formatted_event = format_event(json!(request))?;
     let mut db = init_db()?;
 
     let mut bot = bot_opt.search_bot(&mut db);
@@ -77,12 +79,6 @@ pub fn start_conversation(
         db,
     )?;
 
-    // save event in db as message RECEIVE
-    let msgs = vec![request.payload.to_owned()];
-    if !data.low_data {
-        messages::add_messages_bulk(&mut data, msgs, 0, "RECEIVE")?;
-    }
-
     // block user event if delay variable si on and delay_time is bigger than current time
     if let Some(delay) = bot.no_interruption_delay {
         if let Some(delay) = state::get_state_key(&data.client, "delay", "content", &mut data.db)? {
@@ -109,7 +105,22 @@ pub fn start_conversation(
     }
     //////////////////////////////////////
 
-    check_for_hold(&mut data, &bot)?;
+    check_for_hold(&mut data, &bot, &mut formatted_event)?;
+
+    // save event in db as message RECEIVE
+    match (data.low_data, formatted_event.secure) {
+        (false, true) => {
+            let msgs = vec![serde_json::json!({"content_type": "secure"})];
+
+            messages::add_messages_bulk(&mut data, msgs, 0, "RECEIVE")?;
+        }
+        (false, false) => {
+            let msgs = vec![request.payload.to_owned()];
+
+            messages::add_messages_bulk(&mut data, msgs, 0, "RECEIVE")?;
+        }
+        (true, _) => {}
+    }
 
     let res = interpret_step(&mut data, formatted_event.to_owned(), &bot);
 
@@ -150,11 +161,13 @@ pub fn get_client_messages(
     client: &Client,
     limit: Option<i64>,
     pagination_key: Option<String>,
+    from_date: Option<i64>,
+    to_date: Option<i64>,
 ) -> Result<serde_json::Value, EngineError> {
     let mut db = init_db()?;
     init_logger();
 
-    messages::get_client_messages(client, &mut db, limit, pagination_key)
+    messages::get_client_messages(client, &mut db, limit, pagination_key, from_date, to_date)
 }
 
 pub fn get_client_conversations(
@@ -198,12 +211,17 @@ pub fn create_client_memory(
 /**
  * Create bot version
  */
-pub fn create_bot_version(csml_bot: CsmlBot) -> Result<BotVersionCreated, EngineError> {
+pub fn create_bot_version(mut csml_bot: CsmlBot) -> Result<BotVersionCreated, EngineError> {
     let mut db = init_db()?;
     init_logger();
 
     let bot_id = csml_bot.id.clone();
 
+    // search for modules to download
+    if let Err(err) = search_for_modules(&mut csml_bot) {
+        return Err(EngineError::Interpreter(format!("{:?}", err)));
+    }
+
     match validate_bot(csml_bot.clone()) {
         CsmlResult {
             errors: Some(errors),
@@ -346,10 +364,21 @@ pub fn validate_bot(mut bot: CsmlBot) -> CsmlResult {
                 errors: Some(vec![err]),
                 warnings: None,
                 flows: None,
+                extern_flows: None,
             }
         }
     };
 
+    // search for modules to download
+    if let Err(err) = search_for_modules(&mut bot) {
+        return CsmlResult {
+            errors: Some(vec![ErrorInfo::new(Position::default(), err)]),
+            warnings: None,
+            flows: None,
+            extern_flows: None,
+        };
+    }
+
     csml_interpreter::validate_bot(&bot)
 }
 
@@ -364,9 +393,7 @@ pub fn fold_bot(mut bot: CsmlBot) -> Result<String, EngineError> {
     // load native components into the bot
     bot.native_components = match load_components() {
         Ok(components) => Some(components),
-        Err(err) => {
-            return Err(EngineError::Parring(err.format_error()))
-        }
+        Err(err) => return Err(EngineError::Parring(err.format_error())),
     };
 
     Ok(csml_interpreter::fold_bot(&bot))
@@ -395,7 +422,11 @@ pub fn user_close_all_conversations(client: Client) -> Result<(), EngineError> {
  * If the hold is valid, we also need to load the local step memory
  * (context.hold.step_vars) into the conversation context.
  */
-fn check_for_hold(data: &mut ConversationInfo, bot: &CsmlBot) -> Result<(), EngineError> {
+fn check_for_hold(
+    data: &mut ConversationInfo,
+    bot: &CsmlBot,
+    event: &mut Event,
+) -> Result<(), EngineError> {
     match state::get_state_key(&data.client, "hold", "position", &mut data.db) {
         // user is currently on hold
         Ok(Some(hold)) => {
@@ -419,6 +450,12 @@ fn check_for_hold(data: &mut ConversationInfo, bot: &CsmlBot) -> Result<(), Engi
                 }
             };
 
+            let secure_hold = hold["secure"].as_bool().unwrap_or(false);
+
+            if secure_hold {
+                event.secure = true;
+            }
+
             // all good, let's load the position and local variables
             data.context.hold = Some(Hold {
                 index,
@@ -426,7 +463,9 @@ fn check_for_hold(data: &mut ConversationInfo, bot: &CsmlBot) -> Result<(), Engi
                 step_name: data.context.step.to_owned(),
                 flow_name: data.context.flow.to_owned(),
                 previous: serde_json::from_value(hold["previous"].clone()).unwrap_or(None),
+                secure: secure_hold,
             });
+
             state::delete_state_key(&data.client, "hold", "position", &mut data.db)?;
         }
         // user is not on hold
@@ -457,7 +496,10 @@ pub fn get_status() -> Result<serde_json::Value, EngineError> {
         },
         Err(_) => {
             ready = false;
-            status.insert("database_type".to_owned(), serde_json::json!("error: no database type selected"))
+            status.insert(
+                "database_type".to_owned(),
+                serde_json::json!("error: no database type selected"),
+            )
         }
     };
 
